// ==============================================================
//                 ORBITER MODULE: Scenario Editor
//                  
//
// Editor.cpp
//Aladin Kinach
//aladin_kinach@yahoo.com.br
//www33.brinkster.com/aladinkinach/orbiter
// ==============================================================

#include "Editor.h"
#include <stdio.h>
#include <string.h>
#include <math.h>

HINSTANCE g_hInst;  // module instance handle
DWORD g_dwCmd;      // custom function identifier

char name[256] = "";
OBJHANDLE obj;
int maxVessel, maxBody, maxBase;

// ==============================================================
// Local prototypes
// ==============================================================

void OpenDlgClbk (void *context);
BOOL CALLBACK MsgProc (HWND, UINT, WPARAM, LPARAM);
void ListVessel ( HWND hDlg );
void ListRef ( HWND hDlg );
void ListBase ( HWND hDlg );
bool CreateVessel(HWND hDlg);
void ListClass ( HWND hDlg );
void OpenFile( HWND hDlg );

// ==============================================================
// API interface
// ==============================================================

// ==============================================================
// This function is called when Orbiter starts or when the module
// is activated.

DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	g_hInst = hDLL; // remember the instance handle

	// To allow the user to open our new dialog box, we create
	// an entry in the "Custom Functions" list which is accessed
	// in Orbiter via Ctrl-F4.
	g_dwCmd = oapiRegisterCustomCmd ("Editor",
		"Scenarios Editor.",
		OpenDlgClbk, NULL); 
}

// ==============================================================
// This function is called when Orbiter shuts down or when the
// module is deactivated

DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{
	// Unregister the custom function in Orbiter
	oapiUnregisterCustomCmd (g_dwCmd);
}


// ==============================================================
// Open the dialog window

void OpenDlgClbk (void *context)
{
	HWND hDlg = oapiOpenDialog (g_hInst, IDD_EDITOR, MsgProc);
	// Don't use a standard Windows function like CreateWindow to
	// open the dialog box, because it won't work in fullscreen mode
}

// ==============================================================
// Close the dialog

void CloseDlg (HWND hDlg)
{
	oapiCloseDialog (hDlg);
}

// ==============================================================
// Windows message handler for the dialog box

BOOL CALLBACK MsgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int i;
	char msg[256] = "";

	maxVessel = oapiGetVesselCount();
	maxBody   = oapiGetGbodyCount ();
	
	
	switch (uMsg) {
	
	case WM_INITDIALOG:
		ListVessel ( hDlg );
		ListRef ( hDlg );
		ListBase ( hDlg );
		
		SetDlgItemText (hDlg, ID_LISTCLASS, "DeltaGlider");

		SendDlgItemMessage(hDlg, ID_RADIOBODY, BM_SETCHECK, 1, 0 );
		//enable item
		EnableWindow( GetDlgItem( hDlg, ID_EDITPED ), 1 );
		EnableWindow( GetDlgItem( hDlg, ID_EDITINC ), 1 ); 
				
		//disable item
		EnableWindow( GetDlgItem( hDlg, ID_LISTBASE ), 0 );
		EnableWindow( GetDlgItem( hDlg, ID_EDITLAT ),  0 );
		EnableWindow( GetDlgItem( hDlg, ID_EDITLON ),  0 );
		EnableWindow( GetDlgItem( hDlg, ID_EDITHDG ),  0 );
		
		break;

	case WM_COMMAND:
		switch (LOWORD (wParam)) {

		case ID_RADIOBODY:
			if ( HIWORD (wParam) == BN_CLICKED )
			{
				//disable item
				EnableWindow( GetDlgItem( hDlg, ID_LISTBASE ), 0 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITLAT ),  0 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITLON ),  0 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITHDG ),  0 );
				
				
				//enable item
				EnableWindow( GetDlgItem( hDlg, ID_EDITPED ), 1 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITINC ), 1 ); 
			}
			return true;

		case ID_RADIOLAND:
			if ( HIWORD (wParam) == BN_CLICKED )
			{
				//disable item
				EnableWindow( GetDlgItem( hDlg, ID_EDITPED ), 0 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITINC ), 0 ); 
				EnableWindow( GetDlgItem( hDlg, ID_LISTBASE), 0 );
				
				
				//enable item
				EnableWindow( GetDlgItem( hDlg, ID_EDITLAT ), 1 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITLON ), 1 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITHDG ), 1 );
			}
			return true;

			case ID_RADIOBASE:
			if ( HIWORD (wParam) == BN_CLICKED )
			{
				//disable item
				EnableWindow( GetDlgItem( hDlg, ID_EDITPED ), 0 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITINC ), 0 ); 
				EnableWindow( GetDlgItem( hDlg, ID_EDITLAT ), 0 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITLON ), 0 );
				
				//enable item
				EnableWindow( GetDlgItem( hDlg, ID_LISTBASE), 1 );
				EnableWindow( GetDlgItem( hDlg, ID_EDITHDG ), 1 );
			}
			return true;

		case ID_LISTREF:
			if ( HIWORD (wParam) == CBN_SELCHANGE )
			{
				ListBase ( hDlg );
			}
			return true;

		case IDOK:  // create a new vessel
			if ( CreateVessel( hDlg) )
				MessageBox( hDlg, "Vessel sucessfuly created!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			else
				MessageBox( hDlg, "Failed on create vessel!", "Create Vessel", MB_OK|MB_ICONEXCLAMATION );
			return true;

		case IDCANCEL: // dialog closed by user
			CloseDlg (hDlg);
			return TRUE;

		case ID_DESTROY: //destroy the selected vessel
			GetDlgItemText (hDlg, ID_LISTVESSEL, name, 256);
			obj = oapiGetVesselByName (name);
			
			strcpy( msg, "Delete " );
			strcat( msg, name );
			strcat( msg, "?" );

			i = MessageBox( hDlg, msg, "Delete Vessel", MB_YESNO|MB_ICONQUESTION );
			if ( i == IDYES )
				if ( oapiDeleteVessel(obj) )
				{
					i = SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_SELECTSTRING, 0, (LPARAM) name );
					i = SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_DELETESTRING, i, 0 );
					SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_SETCURSEL, 0, 0 );
				}
			
			return true;

		case ID_SELECT: //select a vessel 
			GetDlgItemText (hDlg, ID_LISTVESSEL, name, 256);
			obj = oapiGetVesselByName (name);
			oapiSetFocusObject(obj);
			return true;

		case ID_OPENFILE: //gets the class of a vessel from config file
			OpenFile( hDlg );
			return true;

		}
		break;
	} 
	return oapiDefDialogProc (hDlg, uMsg, wParam, lParam);
}

void OpenFile( HWND hDlg )
{
	char filename[256], vClass[256],
		filefilter[] = {
		'c', 'o', 'n', 'f', 'i', 'g', '\0', '*', '.', 'c', 'f', 'g', '\0',
		'\0', '\0'};

	OPENFILENAME fname;

	memset( &fname, 0, sizeof(OPENFILENAME));
	fname.lStructSize = sizeof(OPENFILENAME);
	fname.hwndOwner = hDlg;
	fname.lpstrFilter = filefilter;
	fname.nFilterIndex = 1;
	fname.lpstrFileTitle = filename;
	fname.nMaxFileTitle = sizeof(filename) - 1;
	fname.Flags = OFN_HIDEREADONLY;

	if (!GetOpenFileName( &fname ))
		return;

	for (int i = 0; i < strlen(filename) - 4; i++ )
	{
		vClass[i] = filename[i];
		vClass[i+1] = '\0';
	}

	SetDlgItemText (hDlg, ID_LISTCLASS, vClass);
}

void ListVessel ( HWND hDlg )
{
	SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_RESETCONTENT, 0, 0 );
	for ( int i = 0; i < maxVessel; i++ )
	{
		obj = oapiGetVesselByIndex (i);
		oapiGetObjectName (obj, name, 256);
		SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_ADDSTRING, 0, (LPARAM) name );
	}
	//sprintf (oapiDebugString(), "INITDIALOG LISTVESSEL maxVessel %d Nome %s w %d id %d", maxVessel, name, HIWORD (wParam), ID_LISTVESSEL);
	SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_SETCURSEL, 0, 0 );
}

void ListRef ( HWND hDlg)
{
	SendDlgItemMessage(hDlg, ID_LISTREF, CB_RESETCONTENT, 0, 0 );
	for ( int i = 0; i < maxBody; i++ )
	{
		obj = oapiGetGbodyByIndex (i);
		oapiGetObjectName (obj, name, 256);
		SendDlgItemMessage(hDlg, ID_LISTREF, CB_ADDSTRING, 0, (LPARAM) name );
	}
	
	SendDlgItemMessage(hDlg, ID_LISTREF, CB_SETCURSEL, 0, 0 );
	GetDlgItemText (hDlg, ID_LISTREF, name, 256);
}

void ListBase ( HWND hDlg )
{
	GetDlgItemText (hDlg, ID_LISTREF, name, 256);
	
	OBJHANDLE objTemp = oapiGetGbodyByName (name);
	
	SendDlgItemMessage(hDlg, ID_LISTBASE, CB_RESETCONTENT, 0, 0 );
	
	if ( objTemp)
	{
		maxBase = oapiGetBaseCount( objTemp );
		for ( int i = 0; i < maxBase; i++ )
		{
			obj = oapiGetBaseByIndex ( objTemp, i );
			oapiGetObjectName (obj, name, 256);
			SendDlgItemMessage(hDlg, ID_LISTBASE, CB_ADDSTRING, 0, (LPARAM) name );
			//sprintf (oapiDebugString(), "INITDIALOG maxVessel %d Nome %s I %d", maxVessel, name, i);
		}
	}
			
	SendDlgItemMessage(hDlg, ID_LISTBASE, CB_SETCURSEL, 0, 0 );
}

//This method use VESSELSTATUS
/*
bool CreateVessel(HWND hDlg)
{
	VESSELSTATUS vs;
	double lng, lat;
	int status, pad, maxPad;
	char vClass[256], textLat[20], textLon[20], text[256];
	//bool freePad = false;

	GetDlgItemText (hDlg, ID_LISTCLASS, vClass, 256);
	//strcpy(vClass, "ShuttleA");

	GetDlgItemText (hDlg, ID_EDITNEWVESSEL, text, 256);
	if ( SendDlgItemMessage (hDlg, ID_EDITNEWVESSEL, LB_FINDSTRINGEXACT, 0, (LPARAM) text) == LB_ERR)
	{
		MessageBox( hDlg, "A vessel with this name already exists!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
		return false;
	}
	strcpy( name, text );

	vs.flag[0] = 1;
	vs.vrot = _V(0, 0, 0);
	vs.arot = _V(0, 0, 0);

	//set the reference body in vessel status
	GetDlgItemText (hDlg, ID_LISTREF, text, 256);
	obj = oapiGetGbodyByName (text);
	vs.rbody = obj;
	
	if ( SendDlgItemMessage(hDlg, ID_RADIOBODY, BM_GETCHECK, 1, 0 ) )
	{
		VECTOR3 rvel, rpos;
		double R, v, vx, vy, pz, G, M, inc;
		
		try {
			GetDlgItemText (hDlg, ID_EDITPED, text, 256);
			R = atof(text);
			
			GetDlgItemText (hDlg, ID_EDITINC, text, 256);
			inc = atof(text);
			inc /= (180 / PI);
		} 
		catch ( ... ) 
		{
			MessageBox( hDlg, "Invalid value for Raio or Inclination!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false; 
		}

		G = 6.67E-11;
		M = oapiGetMass( vs.rbody );
		R = oapiGetSize( vs.rbody ) + R * 1000;

		v = sqrt ( G * M / R);

		vx = -v * cos( inc );
		vy = v * sin( inc );
		pz = R;

		rvel.x = vx;
		rvel.y = vy;
		rvel.z = 0.0;

		rpos.x = 0.0;
		rpos.y = 0.0;
		rpos.z = pz;

		vs.base = NULL; 
		vs.rvel = rvel;
		vs.rpos = rpos;
		vs.status = 0;
	}
	else
	if ( SendDlgItemMessage(hDlg, ID_RADIOBASE, BM_GETCHECK, 1, 0 ) )
	{
		//set the target base in vessel status
		GetDlgItemText (hDlg, ID_LISTBASE, text, 256);
		obj = oapiGetBaseByName (obj, text);
		vs.base = obj;

		if ( obj )
		{
			maxPad = oapiGetBasePadCount( obj );

			//get a free pad on target base
			for ( pad = 0; pad < maxPad - 1; pad++ )
			{
				oapiGetBasePadStatus (obj, pad, &status);
				if ( status == 0 || status == 2 )
				{
					oapiGetBasePadEquPos ( obj, pad, &lng, &lat);
					break;
				}
			}
		}
		else
		{
			MessageBox( hDlg, "Base don't exists!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false;
		}

		if ( pad < maxPad )
			vs.port = pad;
		else 
		{
			MessageBox( hDlg, "All pads full!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false;
		}

		//sprintf (oapiDebugString(), "Lat %f Long %f", lat, lng);
		
		//set vessel's lat/lon
		vs.vdata[0].x = lat;
		vs.vdata[0].y = lng;
		
		//set vessel's heading
		try {
			GetDlgItemText (hDlg, ID_EDITHDG, text, 256);
			vs.vdata[0].z = atof( text ) * PI / 180;
		}
		catch (...) {
			vs.vdata[0].z = 0;
		}

		vs.status = 1;
	}
	else
	{
		vs.base = NULL; 

		GetDlgItemText (hDlg, ID_EDITLAT, textLat, 256);
		GetDlgItemText (hDlg, ID_EDITLON, textLon, 256);
			
		if (textLat == "" || textLon == "")
		{
			MessageBox( hDlg, "Enter valid decimal latittude and longitude!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false;
		}
		else
		{
			try {
				lat = atof(textLat);
				lat = lat * PI / 180;
				if (toupper( textLat[strlen( textLat ) - 1] ) == 'S')
					lat = -lat;

				lng = atof(textLon);
				lng = lng * PI / 180;
				if (toupper( textLon[strlen( textLon ) - 1] ) == 'W')
					lng = -lng;
			} 
			catch ( ... ) 
			{
				MessageBox( hDlg, "Invalid value for latittude or longittude!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
				return false; 
			}
		}

		//set vessel's lat/lon
		vs.vdata[0].x = lat;
		vs.vdata[0].y = lng;
		
		//set vessel's heading
		try {
			GetDlgItemText (hDlg, ID_EDITHDG, text, 256);
			vs.vdata[0].z = atof( text ) * PI / 180;
		}
		catch (...) {
			vs.vdata[0].z = 0;
		}

		vs.status = 1;
	}


	sprintf (oapiDebugString(), "Classe %s", vClass);
	MessageBox( hDlg, vClass, "Create Vessel", MB_OK|MB_ICONINFORMATION );
	//create the new vessel using a name given by user
	if ( name != NULL)
		obj = oapiCreateVesselEx (name, vClass, &vs);
	else
		obj = NULL;

	if ( obj )
	{
		ListVessel(hDlg); //reset list
		return true;
	}
	else
		return false;
}
*/

//This method use VESSELSTATUSX
//I use the two structs to try create another vessel's class 
//but it only works with Delta Glider
bool CreateVessel(HWND hDlg)
{
	VESSELSTATUS2 vs;
	double lng, lat;
	int status, pad, maxPad;
	char vClass[256], textLat[20], textLon[20], text[256];
	//bool freePad = false;

	vs.fuel = NULL;
	vs.thruster = NULL;
	vs.xpdr = 0;
	vs.version = 2;
	vs.flag = VS_THRUSTRESET;
	vs.vrot = _V(0, 0, 0);
	vs.arot = _V(0, 0, 0);

	GetDlgItemText (hDlg, ID_LISTCLASS, vClass, 256);
	//strcpy(vClass, "ShuttleA");

	GetDlgItemText (hDlg, ID_EDITNEWVESSEL, text, 256);
	if ( SendDlgItemMessage (hDlg, ID_EDITNEWVESSEL, LB_FINDSTRINGEXACT, 0, (LPARAM) text) == LB_ERR)
	{
		MessageBox( hDlg, "A vessel with this name already exists!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
		return false;
	}
	strcpy( name, text );

	//set the reference body in vessel status
	GetDlgItemText (hDlg, ID_LISTREF, text, 256);
	obj = oapiGetGbodyByName (text);
	vs.rbody = obj;
	
	if ( SendDlgItemMessage(hDlg, ID_RADIOBODY, BM_GETCHECK, 1, 0 ) )
	{
		VECTOR3 rvel, rpos;
		double R, v, vx, vy, pz, G, M, inc;
		
		try {
			GetDlgItemText (hDlg, ID_EDITPED, text, 256);
			R = atof(text);
			
			GetDlgItemText (hDlg, ID_EDITINC, text, 256);
			inc = atof(text);
			inc /= (180 / PI);
		} 
		catch ( ... ) 
		{
			MessageBox( hDlg, "Invalid value for Raio or Inclination!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false; 
		}

		G = 6.67E-11;
		M = oapiGetMass( vs.rbody );
		R = oapiGetSize( vs.rbody ) + R * 1000;

		v = sqrt ( G * M / R);

		vx = -v * cos( inc );
		vy = v * sin( inc );
		pz = R;

		rvel.x = vx;
		rvel.y = vy;
		rvel.z = 0.0;

		rpos.x = 0.0;
		rpos.y = 0.0;
		rpos.z = pz;

		vs.base = NULL; 
		vs.rvel = rvel;
		vs.rpos = rpos;
		vs.status = 0;
	}
	else
	if ( SendDlgItemMessage(hDlg, ID_RADIOBASE, BM_GETCHECK, 1, 0 ) )
	{
		//set the target base in vessel status
		GetDlgItemText (hDlg, ID_LISTBASE, text, 256);
		obj = oapiGetBaseByName (obj, text);
		vs.base = obj;

		if ( obj )
		{
			maxPad = oapiGetBasePadCount( obj );

			//get a free pad on target base
			for ( pad = 0; pad < maxPad - 1; pad++ )
			{
				oapiGetBasePadStatus (obj, pad, &status);
				if ( status == 0 || status == 2 )
				{
					oapiGetBasePadEquPos ( obj, pad, &lng, &lat);
					break;
				}
			}
		}
		else
		{
			MessageBox( hDlg, "Base don't exists!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false;
		}

		if ( pad < maxPad )
			vs.port = pad;
		else 
		{
			MessageBox( hDlg, "All pads full!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false;
		}

		//sprintf (oapiDebugString(), "Lat %f Long %f", lat, lng);
		
		//set vessel's lat/lon
		vs.surf_lng = lng;
		vs.surf_lat = lat;
		
		//set vessel's heading
		try {
			GetDlgItemText (hDlg, ID_EDITHDG, text, 256);
			vs.surf_hdg = atof( text ) * PI / 180;
		}
		catch (...) {
			vs.surf_hdg = 0;
		}

		vs.status = 1;
	}
	else
	{
		vs.base = NULL; 

		GetDlgItemText (hDlg, ID_EDITLAT, textLat, 256);
		GetDlgItemText (hDlg, ID_EDITLON, textLon, 256);
			
		if (textLat == "" || textLon == "")
		{
			MessageBox( hDlg, "Enter valid decimal latittude and longitude!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
			return false;
		}
		else
		{
			try {
				lat = atof(textLat);
				lat = lat * PI / 180;
				if (toupper( textLat[strlen( textLat ) - 1] ) == 'S')
					lat = -lat;

				lng = atof(textLon);
				lng = lng * PI / 180;
				if (toupper( textLon[strlen( textLon ) - 1] ) == 'W')
					lng = -lng;
			} 
			catch ( ... ) 
			{
				MessageBox( hDlg, "Invalid value for latittude or longittude!", "Create Vessel", MB_OK|MB_ICONINFORMATION );
				return false; 
			}
		}

		//set vessel's lat/lon
		vs.surf_lng = lng;
		vs.surf_lat = lat;
		
		//set vessel's heading
		try {
			GetDlgItemText (hDlg, ID_EDITHDG, text, 256);
			vs.surf_hdg = atof( text ) * PI / 180;
		}
		catch (...) {
			vs.surf_hdg = 0;
		}

		vs.status = 1;
	}


	//sprintf (oapiDebugString(), "Classe %s", vClass);
	//MessageBox( hDlg, vClass, "Create Vessel", MB_OK|MB_ICONINFORMATION );
	//create the new vessel using a name given by user
	if ( name != NULL)
		obj = oapiCreateVesselEx (name, vClass, &vs);
	else
		obj = NULL;

	if ( obj )
	{
		ListVessel(hDlg); //reset list
		return true;
	}
	else
		return false;
}


void ListClass ( HWND hDlg )
{
	ifstream ent;
	HANDLE hFiles;
	LPWIN32_FIND_DATA proxArq = NULL;
	char arquivo[256], path[256] = "..\\..\\Config\\";
//	char classVessel[256];
	BOOL *dir = false;

	/*strcpy( arquivo, path );
	strcat( arquivo, "*.*" );*/
	strcpy( arquivo, "*.*" );

	MessageBox( hDlg, arquivo, "Teste", MB_OK);
	hFiles = FindFirstFile( arquivo, proxArq );
	MessageBox( hDlg, proxArq->cFileName, "File Name", MB_OK);

	/*
	do
	{
		
		strcpy( arquivo, path );
		strcat( arquivo, proxArq->cFileName);
		if(! (proxArq->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
		//ent.open ( arquivo, ios::nocreate, 1);
		//if (ent)
		{
			
			cin >> classVessel;
			MessageBox( hDlg, classVessel, "Teste", MB_OK);
			ent.close();
			
			MessageBox( hDlg, proxArq->cFileName , "Proximo File Name", MB_OK);
		}
		//else
			//MessageBox( hDlg, "erro abrir arquivo", "Teste", MB_OK);
	}
	while ( FindNextFile( hFiles, proxArq ) );

	SendDlgItemMessage(hDlg, ID_LISTCLASS, CB_RESETCONTENT, 0, 0 );
	for ( int i = 0; i < maxVessel; i++ )
	{
		obj = oapiGetVesselByIndex (i);
		oapiGetObjectName (obj, name, 256);
		SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_ADDSTRING, 0, (LPARAM) name );
	}
	//sprintf (oapiDebugString(), "INITDIALOG LISTVESSEL maxVessel %d Nome %s w %d id %d", maxVessel, name, HIWORD (wParam), ID_LISTVESSEL);
	SendDlgItemMessage(hDlg, ID_LISTVESSEL, CB_SETCURSEL, 0, 0 );
*/
}


//I had tryed this one, but I can't make it works with VESSELSTATUSX
void ele2vec (VECTOR3 rpos, VECTOR3 rvel, double sma, double ecc, double inc, 
			  double ap, double ran, double M)
{
	double a=sma;				//semi major axis
	double ec=ecc;				//eccentricity
	double i=(inc/(180 / PI));	//inclination
	double w0=(ap/(180 / PI));	//argument of perigee
	double o0=(ran/(180 / PI));	//Right ascend of ascend node
	double m0=(M/(180 / PI));	//Mean anomaly
	double J2=.001082;
	double re=6378.16;
	double n0= sqrt(398601.3/(a*a*a));
	double eca= m0+ec/2.0;
	double diff=10000.0;
	double eps= 0.000001;
	double ceca, seca, edot;
	double xdw, ydw;
	double e1, xw, yw, cw, sw, co, so, ci, si, swci, cwci, px, py, pz, qx, qy, qz;
	double x, y, z, xd, yd, zd;

	while(diff>eps)
	{
		e1=eca-(eca-ec*sin(eca)-m0)/(1-ec*cos(eca));
		diff=/*abs*/(e1-eca);
		eca=e1;
	}

	ceca=cos(eca);
	seca=sin(eca);
	e1=a*sqrt(1-ec*ec);
	xw=a*(ceca-ec); 
	yw=e1*seca;
	edot=sqrt(398601.3/a)/(a*(1-ec*ceca));
	xdw=-a*edot*seca;
	ydw=e1*edot*ceca;
	cw = cos(w0); 
	sw=sin(w0);
	co=cos(o0);
	so=sin(o0);
	ci=cos(i); 
	si=sin(i);
	swci=sw*ci;
	cwci=cw*ci;
	px=cw*co-so*swci;
	py=cw*so+co*swci;
	pz=sw*si;
	qx=-sw*co-so*cwci;
	qy=-sw*so+co*cwci;
	qz=cw*si;

	//rpos
	x=xw*px+yw*qx;
	y=xw*py+yw*qy; 
	z=xw*pz+yw*qz;

	//rvel
	xd=xdw*px+ydw*qx;
	yd=xdw*py+ydw*qy;
	zd=xdw*pz+ydw*qz;
}
